//
//  ABOAssetSampleEnumerator.m
//  Magi
//
//  Created by Jamie Hardt on 7/4/12.
//  Copyright (c) 2012 Jamie Hardt. All rights reserved.
//

#import "ABOAssetSampleEnumerator.h"


@implementation ABOAssetSampleEnumerator 

@synthesize timeRange = _timeRange;

-(id)initWithAsset:(AVAsset *)asset
             track:(AVAssetTrack *)track
    outputSettings:(NSDictionary *)settings {
    if (self = [super init] ) {
        
        _asset = asset;
        _track = track;
        _settings = settings;
        _readerLock = [NSLock new];
        
        _timeRange = CMTimeRangeMake(kCMTimeZero, kCMTimePositiveInfinity);

        return self;
    } else {
        return nil;
    }
}

-(id)initWithAsset:(AVAsset *)asset
             track:(AVAssetTrack *)track {
    return [self initWithAsset:asset track:track outputSettings:nil];
}

-(void)enumerateSamplesWithBlock:(void(^)(CMSampleBufferRef buffer))sampleBlock 
                    errorHandler:(void(^)(NSError *error))errorBlock
                   finishHandler:(void(^)(AVAssetReaderStatus status))finishBlock {
    
    [_readerLock lock];
    NSError *err = nil;
    AVAssetReader *reader = [AVAssetReader assetReaderWithAsset:_asset
                                error:&err];
    
    if (err) {
        errorBlock(err);
    } else {
        AVAssetReaderTrackOutput *output = [AVAssetReaderTrackOutput assetReaderTrackOutputWithTrack:_track outputSettings:_settings];
        
        [reader addOutput:output];
        reader.timeRange = _timeRange;
        
        if ([reader startReading]) {
            CMSampleBufferRef aBuffer = NULL;
            while ((aBuffer = [output copyNextSampleBuffer]) != NULL) {
                
                sampleBlock(aBuffer);
                
                CFRelease(aBuffer);
            }
            finishBlock([reader status]);
            
            
        } else {
            errorBlock([reader error]);
        }
    }


    [_readerLock unlock];
}


@end
