//
//  ABOComplexVector.m
//  Magi
//
//  Created by Jamie Hardt on 4/29/12.
//  Copyright (c) 2012 Jamie Hardt. All rights reserved.
//

#import "ABOComplexVector.h"

#define ASSERT_BOUNDS(pos,ch) do{\
NSAssert(chan < _priv->channels, @"channel index (%u) out of bounds (%u) of vector",chan,_priv->channels);\
NSAssert(position < _priv->length/2+1, @"position index (%u) out of bounds (%u) of vector",pos, _priv->length/2+1);\
}while(0);

@implementation ABOComplexVector

@synthesize CVec = _priv;

#pragma mark Initializers

-(id)initWithLength:(uint_t)length
       channelCount:(uint_t)channels {
    cvec_t *cv = new_cvec(length, channels);
    if (cv) {
        self = [self initWithCvec:cv];
    } else {
        self = nil;
    }
    return self;
}

-(id)initWithCvec:(cvec_t *)complexVector {
    if (self = [super init] ) {
        _priv = complexVector;
    }
    return self;
}

#pragma mark -

-(void)dealloc {
    del_cvec(_priv);
}

#pragma mark -

#pragma mark Properties

-(uint_t)length {
    return _priv->length;
}

-(uint_t)channelCount {
    return _priv->channels;
}

#pragma mark Work with Samples

-(void)setNorm:(smpl_t)normVal phase:(smpl_t)phaseVal
    atPosition:(uint_t)position inChannel:(uint_t)chan {
    
    ASSERT_BOUNDS(position, chan);
    
    _priv->norm[chan][position] = normVal;
    _priv->phas[chan][position] = phaseVal;
    
}

-(void)getNorm:(smpl_t *)normVal phase:(smpl_t *)phaseVal 
 atPosition:(uint_t)position inChannel:(uint_t)chan {
    
    *normVal = 0, *phaseVal = 0;
    
    ASSERT_BOUNDS(position,chan);
    
    *normVal = _priv->norm[chan][position];
    *phaseVal = _priv->phas[chan][position];
}

@end
