//
//  ABOCoreAudioHelpers.c
//  Magi
//
//  Created by Jamie Hardt on 4/14/12.
//  Copyright (c) 2012 Jamie Hardt. All rights reserved.
//


#include "ABOCoreAudioHelpers.h"


/*  This function is from https://github.com/sbooth/SFBAudioEngine/blob/master/AllocateABL.cpp  */

AudioBufferList *_CreateBufferList(UInt32 channelsPerFrame, 
                             UInt32 bytesPerFrame, 
                             Boolean interleaved, 
                             UInt32 capacityFrames) {
	AudioBufferList *bufferList = NULL;
    
	UInt32 numBuffers = interleaved ? 1 : channelsPerFrame;
	UInt32 channelsPerBuffer = interleaved ? channelsPerFrame : 1;
    
	bufferList = (AudioBufferList *)(calloc(1, offsetof(AudioBufferList, mBuffers) + (sizeof(AudioBuffer) * numBuffers)));
    
	bufferList->mNumberBuffers = numBuffers;
    
    UInt32 bufferIndex;
	for(bufferIndex = 0; bufferIndex < bufferList->mNumberBuffers; ++bufferIndex) {
		bufferList->mBuffers[bufferIndex].mData = calloc(capacityFrames, bytesPerFrame);
		bufferList->mBuffers[bufferIndex].mDataByteSize = capacityFrames * bytesPerFrame;
		bufferList->mBuffers[bufferIndex].mNumberChannels = channelsPerBuffer;
	}
    
	return bufferList;
}

void DestroyBufferList(AudioBufferList *aList) {
    
    UInt32 i;
    for (i = 0; i < aList->mNumberBuffers; i++) {
        free(aList->mBuffers[i].mData);
    }
    free(aList);
}

AudioBufferList *CreateBufferList(const AudioStreamBasicDescription *format, 
                                  UInt32 capacityFrames) {

	return _CreateBufferList(format->mChannelsPerFrame, 
                             format->mBytesPerFrame, 
                             !(kAudioFormatFlagIsNonInterleaved & format->mFormatFlags), capacityFrames);
}
