//
//  ABOOnsetDetector.m
//  Magi
//
//  Created by Jamie Hardt on 7/6/12.
//  Copyright (c) 2012 Jamie Hardt. All rights reserved.
//

#import "ABOOnsetDetector.h"
#import "ABOVectorEnumerator.h"
#import <aubio/aubio.h>
#import "ABOSampleVector.h"

@implementation ABOOnsetDetector

-(id)initWithAsset:(AVAsset *)asset track:(AVAssetTrack *)track {
    if (self = [super init]) {
        NSAssert(asset && track,@"asset and track must not be nil");
        _asset = asset;
        _track = track;
        return self;
    } else {
        return nil;
    }
}

- (NSArray *)_processOnsetVector:(ABOSampleVector *)onsetVector 
                     sampleRate:(int32_t)sampleRate 
                         offset:(NSUInteger)offset 
                        channel:(NSUInteger)channel {
    
    NSMutableArray *retObj = [NSMutableArray array];
    
    smpl_t val = 0.0f;
    CMTime lastTime = CMTimeMake(0, sampleRate);
    int i;
    for (i = 0; i < onsetVector.length; i++) {
        val = [onsetVector sampleAtFrame:i inChannel:channel];
        if (val > 0.0f) {
            CMTime aTime = CMTimeMake(i+offset, sampleRate);
            if (CMTimeCompare(lastTime, aTime) == -1) {
                [retObj addObject:[NSValue valueWithCMTime:aTime]];
                lastTime = aTime;
            }
           
        } 
    }
    return [NSArray arrayWithArray:retObj];
}

-(NSArray *)onsetsOfType:(aubio_onsetdetection_type)type 
               onChannel:(NSUInteger)channel
           withPeakThreshold:(smpl_t)threshold {    
    
    NSMutableArray *retArray = [NSMutableArray array];;
    ABOVectorEnumerator *v = [[ABOVectorEnumerator alloc] initWithAsset:_asset
                                                                  track:_track];
    
    NSAssert(v,@"failed to create a sample enumerator "
             "for asset (%@) and track (%@)",
             _asset,_track);
    
    NSAssert(channel < v.assetChannelCount,@"index of channel %u is greater "
             "than asset channel count %u",channel, v.assetChannelCount);
    
    v.windowSize = 512; v.periodLength = 256;
    
    aubio_onset_t *onsetDetector = new_aubio_onset(type, v.windowSize, v.periodLength, v.assetChannelCount);
    
    
    __block NSUInteger offset = 0; 
    
    [v enumerateSamplesWithBlock:^(ABOSampleVector *vector) {
        ABOSampleVector *onsetVector = [[ABOSampleVector alloc] initWithLength:vector.length
                                                                  channelCount:vector.channelCount];
        
        aubio_onset(onsetDetector, [vector FVec], [onsetVector FVec]);
        aubio_onset_set_threshold(onsetDetector, threshold);
        
        int32_t sampleRate = floor([v assetSampleRate]);
        NSArray *toAdd = [self _processOnsetVector:onsetVector sampleRate:sampleRate offset:offset channel:channel];
        if (toAdd && [toAdd count]) {
          [retArray addObjectsFromArray:toAdd];    
        }
        

        
        offset += v.periodLength;
    } 
                    errorHandler:^(NSError *error) {
        
    }];
    
  
    return [NSArray arrayWithArray:retArray];
}

@end
