//
//  ABOFloatVector.h
//  Magi
//
//  Created by Jamie Hardt on 4/8/12.
//  Copyright (c) 2012 Jamie Hardt. All rights reserved.
//

/*
 This file is part of Magi.
 
 Magi is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, version 2.0.
 
 Magi is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Magi.  If not, see <http://www.gnu.org/licenses/>.
 */

#import <Cocoa/Cocoa.h>
#import <CoreMedia/CoreMedia.h>
#import <CoreAudio/CoreAudio.h>
#import <aubio/aubio.h>

@interface ABOSampleVector : NSObject <NSCopying, NSCoding> {
    fvec_t *_priv;
    BOOL _privFreeWhenDone;
}

/*  Initializers */ 

-(id)initWithLength:(uint_t)length 
       channelCount:(uint_t)channels;

-(id)initWithFvec:(fvec_t *)vector;

-(id)initWithFvecNoCopy:(fvec_t *)vector freeWhenDone:(BOOL)freeWhenDone;

-(id)initWithCMSampleBuffer:(CMSampleBufferRef)sampleBuffer;

-(id)initWithAudioBufferList:(AudioBufferList)bufferList 
 audioStreamBasicDescription:(AudioStreamBasicDescription)absd;



/* Properties */

@property (readonly) fvec_t *FVec;
@property (readonly) uint_t length;
@property (readonly) uint_t channelCount;


/* Vector Composition */

-(id)vectorByAppendingVector:(ABOSampleVector *)vector;

-(id)vectorByTrimmingFromSample:(uint_t)offset 
                       length:(uint_t)length;

-(NSArray *)subvectorsOfLength:(NSUInteger)length
                        period:(NSUInteger)period;

/* Sample access */

-(smpl_t)sampleAtFrame:(uint_t)frame inChannel:(uint_t)chan;

-(void)setSample:(smpl_t)sampleValue 
         atFrame:(uint_t)frame inChannel:(uint_t)chan;

-(smpl_t *)getChannel:(uint_t)channel;

-(void)putSamples:(smpl_t *)samples withLength:(uint_t)length
        inChannel:(uint_t)channel;

-(void)putSamples:(smpl_t *)samples withLength:(uint_t)length
         atOffset:(uint_t)offset 
        inChannel:(uint_t)channel;


@end
