//
//  ABOTestSignalGenerator.m
//  Magi
//
//  Created by Jamie Hardt on 4/29/12.
//  Copyright (c) 2012 Jamie Hardt. All rights reserved.
//

#import "ABOTestSignalGenerator.h"

#define PI  3.1415926f

static ABOTestSignalGenerator *_singletonTSG;

@implementation ABOTestSignalGenerator

@synthesize  peakGain = _peakGain, dutyCycle = _dutyCycle;

+(void)initialize {
    _singletonTSG = nil;
}

+(id)testGenerator {
    
    if (!_singletonTSG) {
        _singletonTSG = [[ABOTestSignalGenerator alloc] init];
    }
    
    return _singletonTSG;
}

-(id)init {
    if (self = [super init]) {
        _peakGain = 1.0f;
        _dutyCycle = 0.5f;
        _signalType = ABOSignalTypeSineWave;
    } else {
        self = nil;
    }
    
    return self;
}

-(ABOSampleVector *)createTestSignalWithPeriods:(uint_t)length
                               samplesPerPeriod:(uint_t)samples {
    
    NSAssert(samples > 2 && length > 0 ,@"periods (%u) must be "
             "greater than 0 and less than length/2");
    
    ABOSampleVector *retObj = nil;
    
    fvec_t *work = new_fvec(length * samples, 1);
    smpl_t *oneCycleBuf = calloc(samples, sizeof(smpl_t));
    uint_t i;
    
    switch (_signalType) {
        case ABOSignalTypeSineWave:
            for (i = 0; i < samples; i++) {
                oneCycleBuf[i] = _peakGain * sinf((float)i / (float)samples * 2 * PI);
            }
            break;
            
        case ABOSignalTypeSquareWave:
            for (i = 0; i < samples; i++) {
                oneCycleBuf[i] = (i < samples * _dutyCycle ? _peakGain : _peakGain * -1);
            }
            break;
            
        case ABOSignalTypeDiracComb:
            oneCycleBuf[0] = _peakGain;
            break;
            
        case ABOSignalTypeWhiteNoise:
            for (i = 0; i < samples; i++) {
                oneCycleBuf[i] = _peakGain * ((float)rand() / (float)RAND_MAX);
            }
            break;
            
        default:
            NSAssert(0,@"invalid signal type");
            break;
    }
    
    for (i = 0; i < length; i++) {
        memcpy(work->data[0] + (samples * i), 
               oneCycleBuf, 
               samples * sizeof(smpl_t) );
    }
    
    retObj = [[ABOSampleVector alloc] initWithFvec:work];
    del_fvec(work);
    free(oneCycleBuf);
    return retObj;
}


@end
