//
//  ABOAssetVectorEnumerator.m
//  Magi
//
//  Created by Jamie Hardt on 7/8/12.
//  Copyright (c) 2012 Jamie Hardt. All rights reserved.
//

#import "ABOVectorEnumerator.h"
#import "ABOAssetSampleEnumerator.h"
#import "ABOSampleVector.h"

@implementation ABOVectorEnumerator

@synthesize windowSize = _windowSize, periodLength = _periodLength,
timeRange = _timeRange;

+(NSDictionary *)outputSettingsForSampleEnumerator {
    return [NSDictionary dictionaryWithObjectsAndKeys:
            
            [NSNumber numberWithUnsignedInt:'lpcm'], 
            AVFormatIDKey,
            
            [NSNumber numberWithInt:32],
            AVLinearPCMBitDepthKey,
            
            [NSNumber numberWithBool:YES],
            AVLinearPCMIsFloatKey,
            
            [NSNumber numberWithBool:NO],
            AVLinearPCMIsNonInterleaved,
            nil];
}

-(id)initWithAsset:(AVAsset *)asset
             track:(AVAssetTrack *)track {
    self = [super init];
    if (self) {
        NSAssert(asset,@"initWithAsset called with a nil asset");
        NSAssert(track,@"initWithAsset called with a nil track");
        NSAssert(
        [[asset tracksWithMediaType:AVMediaTypeAudio] containsObject:track],
                 @"asset %@ does not contain track %@",asset, track);
        _asset = asset;
        _track = track;
        _windowSize = 512;
        _periodLength = 256;
        
        _timeRange = CMTimeRangeMake(kCMTimeZero, kCMTimePositiveInfinity);
        
        CMFormatDescriptionRef format = (__bridge_retained CMFormatDescriptionRef)[[_track formatDescriptions] lastObject];
        
        const AudioStreamBasicDescription *absd;
        absd = CMAudioFormatDescriptionGetStreamBasicDescription(format);
        
        NSAssert(absd != NULL, @"invalid audio track, no AudioStreamBasicDescription");

        _audioStreamBasicDescription = *absd;
        CFRelease(format);
        
    }
    return self;
}

-(double)assetSampleRate {
    
    return _audioStreamBasicDescription.mSampleRate;
}


- (NSUInteger)assetChannelCount {
    
    return _audioStreamBasicDescription.mChannelsPerFrame;
}

-(void)enumerateSamplesWithBlock:(void(^)(ABOSampleVector *))sampleBlock
                    errorHandler:(void(^)(NSError *error))errorBlock {
    
    ABOAssetSampleEnumerator *sampleEnum = 
    [[ABOAssetSampleEnumerator alloc] initWithAsset:_asset 
                                              track:_track
                                     outputSettings:[[self class] outputSettingsForSampleEnumerator]];
    
    sampleEnum.timeRange = _timeRange;
    
    NSUInteger channelCount;
    channelCount = [self assetChannelCount];
    
    __block ABOSampleVector *workVector = [[ABOSampleVector alloc] initWithLength:0 channelCount:channelCount];
    __block ABOSampleVector *yieldVector = nil;
    __block enum {windowing,striding} state = windowing;
    
    // copy these so the caller can't modify them once reading has started
    NSUInteger thisWindowSize = _windowSize;
    NSUInteger thisPeriodLength = _periodLength;
    
    // I really fucked this up....  it works but it should be much simpler
    
    [sampleEnum enumerateSamplesWithBlock:^(CMSampleBufferRef buffer) {
        ABOSampleVector *bufferVector = [[ABOSampleVector alloc] initWithCMSampleBuffer:buffer];
        
        workVector = [workVector vectorByAppendingVector:bufferVector];
        
        if (state == windowing) {
            if (workVector.length >= thisWindowSize) {
                yieldVector = [workVector vectorByTrimmingFromSample:0 length:thisWindowSize];
                sampleBlock(yieldVector);
                state = striding;
            }
        } else if (state == striding) {
            if (workVector.length >= thisPeriodLength) {
                workVector = [workVector vectorByTrimmingFromSample:thisPeriodLength 
                                                             length:workVector.length - thisPeriodLength];
                state = windowing;
            }
        }

    }
                             
                             errorHandler:^(NSError *error) {
                                 errorBlock(error);
                             } 
                            
                            finishHandler:^(AVAssetReaderStatus status) {
                                
                                if (status != AVAssetReaderStatusCancelled) {
                                    // purge the rest of the buffer
                                    
                                    do {
                                        if (state == windowing) {
                                            if (workVector.length >= thisWindowSize) {
                                                yieldVector = [workVector vectorByTrimmingFromSample:0 length:thisWindowSize];
                                                sampleBlock(yieldVector);
                                                state = striding;
                                            } else {
                                                ABOSampleVector *fillVector = 
                                                [[ABOSampleVector alloc] initWithLength:thisWindowSize - workVector.length 
                                                                           channelCount:channelCount];
                                                yieldVector = [workVector vectorByAppendingVector:fillVector];
                                                sampleBlock(yieldVector);
                                                break;
                                            }
                                        } else if (state == striding) {
                                            if (workVector.length >= thisPeriodLength) {
                                                workVector = [workVector vectorByTrimmingFromSample:thisPeriodLength 
                                                                                             length:workVector.length - thisPeriodLength];
                                                state = windowing;
                                            } else {
                                                break;
                                            }
                                        }
                                    } while (1);
                                    
                                }
                            }];
    
}

@end
