//
//  ABOIntegration_PitchDetection.m
//  Magi
//
//  Created by Jamie Hardt on 6/29/12.
//  Copyright (c) 2012 Jamie Hardt. All rights reserved.
//
#import <aubio/aubio.h>
#import <AVFoundation/AVFoundation.h>
#import <CoreMedia/CoreMedia.h>
#import "ABOIntegration_PitchDetection.h"

@implementation ABOIntegration_PitchDetection

-(void)testWithSignalGenerator {
    
    ABOTestSignalGenerator *tsg = [ABOTestSignalGenerator testGenerator];
    
    ABOSampleVector *oneSec1kTone = [tsg createTestSignalWithPeriods:1000
                                                    samplesPerPeriod:48];
    
    ABOSampleVector *oneSec440Tone = [tsg createTestSignalWithPeriods:440 
                                                     samplesPerPeriod:109];
    
    
    aubio_pitchdetection_t *pitchDetector = new_aubio_pitchdetection(48000, 
                                                                     48000, 
                                                                     1, 
                                                                     48000,

                                                                     aubio_pitch_fcomb , aubio_pitchm_freq);
    
    
    smpl_t freq = aubio_pitchdetection(pitchDetector, 
                                      [oneSec1kTone FVec] );
    
    
    STAssertEqualsWithAccuracy(freq, 1000.0f , 20.0f,@"pitch detection failed, should have been 1000 within 2%, was %f",freq);
    
    freq = aubio_pitchdetection(pitchDetector, [oneSec440Tone FVec]);
    
    STAssertEqualsWithAccuracy(freq, 440.0f, 8.8f, @"pitch detection failed, should have been 440 within 2%, was %f",freq);
    
    
    del_aubio_pitchdetection(pitchDetector);
}

-(void)testWithSignalFile {
    AVAsset *asset = [AVAsset assetWithURL:[[NSBundle bundleForClass:[self class]] URLForResource:@"1k Tone" 
                                                                                    withExtension:@"wav"]];
    
    if (asset) {
        NSError *err = nil;
        AVAssetReader *reader = [AVAssetReader assetReaderWithAsset:asset
                                                              error:&err];
        STAssertNil(err,@"could not create reader for test, got error: %@",err);
        
        AVAssetReaderOutput *output = [AVAssetReaderTrackOutput assetReaderTrackOutputWithTrack:[[asset tracks] lastObject] 
                                                                                 outputSettings:nil];
        [reader addOutput:output];
        
        if ([reader startReading]) {
            CMSampleBufferRef buffer = NULL;
            buffer = [output copyNextSampleBuffer];
            
            CMItemCount size = CMSampleBufferGetNumSamples(buffer);
            
            aubio_pitchdetection_t *detector = new_aubio_pitchdetection(size, 
                                                                        size, 1, 48000, 
                                                                        aubio_pitch_fcomb, 
                                                                        aubio_pitchm_freq);
            
            ABOSampleVector *samples = [[ABOSampleVector alloc] initWithCMSampleBuffer:buffer];
            
            smpl_t freq = aubio_pitchdetection(detector, [samples FVec]);
            
            STAssertEqualsWithAccuracy(freq, 1000.0f, 20.0f,
                                       @"reading freq of file not accurate withing 2%");
            CFRelease(buffer);
            [reader cancelReading];
        } else {
            STAssertFalse(YES,@"reader failed to start reading");
        }
    }
}

@end
