//
//  ABOSampleBuffer_Arithmetic_testCase.m
//  Magi
//
//  Created by Jamie Hardt on 7/8/12.
//  Copyright (c) 2012 Jamie Hardt. All rights reserved.
//

#import "ABOSampleBuffer_Arithmetic_testCase.h"


//-(smpl_t)maxAtSample:(uint_t *)index inChannel:(uint_t *)channel;
//-(smpl_t)minAtSample:(uint_t *)index inChannel:(uint_t *)channel;

#define FLOAT_TOLERANCE (0.000001f)

@implementation ABOSampleBuffer_Arithmetic_testCase

-(void)setUp {
    _test = [[ABOSampleVector alloc] initWithLength:10 channelCount:2];
    
    smpl_t *bufCh1, *bufCh2;
    bufCh1 = malloc(sizeof(smpl_t) * 10);
    bufCh2 = malloc(sizeof(smpl_t) * 10);
    
    int i;
    for (i = 0; i < 10; i++) {
        bufCh1[i] = 0.2f + 0.1f * i;
        bufCh2[i] = 0.3f - 0.1f * i;
    }
    [_test putSamples:bufCh1 withLength:10 inChannel:0];
    [_test putSamples:bufCh2 withLength:10 inChannel:1];
    
    free(bufCh1);
    free(bufCh2);
}

-(void)testMax {
    smpl_t buf[] = {0.1f,0.1f,1.0f,0.0f,0.1f,1.0f,0.1f,0.0f,0.0f,0.0f};
    ABOSampleVector *t = [[ABOSampleVector alloc] initWithLength:10 channelCount:1];
    [t putSamples:buf withLength:10 inChannel:0];
    
    uint_t myIndex, myChannel;
    
    smpl_t val = [t peakAtFrame:&myIndex inChannel:&myChannel];
    
    STAssertEquals((smpl_t)1.0f, val,@"max sample value is wrong");
    STAssertEquals((uint_t) 2, myIndex,@"index is incorrect");
    STAssertEquals((uint_t) 0, myChannel,@"channel is incorrect");
    
}

-(void)testSum {
    STAssertEqualsWithAccuracy([_test sum], 5.0f, FLOAT_TOLERANCE, @"sum failed");
}

-(void)testRectfifiedSum {
    STAssertEqualsWithAccuracy([_test rectifiedSum], (smpl_t)9.2f, FLOAT_TOLERANCE  ,@"rectifiedSum failed");
}

-(void)testAverage {
    STAssertEqualsWithAccuracy([_test average], (smpl_t)0.25f, FLOAT_TOLERANCE , @"average failed");
}

-(void)testRectifiedAverage {
    STAssertEqualsWithAccuracy([_test rectifiedAverage], (smpl_t)0.46f,  FLOAT_TOLERANCE , @"rectifiedAverage failed");    
}

-(void)testMono {
    ABOSampleVector *m = [_test monoVector];
    
    int i;
    for (i = 0; i < m.length; i++) {
        STAssertEqualsWithAccuracy([m sampleAtFrame:i inChannel:0], 0.5f, FLOAT_TOLERANCE,@"mono failed");
    }
}

@end
