//
//  ABOFloatVector_testCase.m
//  Magi
//
//  Created by Jamie Hardt on 4/9/12.
//  Copyright (c) 2012 Jamie Hardt. All rights reserved.
//

#import "ABOSampleVector_testCase.h"


@implementation ABOSampleVector_testCase

-(void)setUp {
    testVector = new_fvec(8, 2);
    
    smpl_t buf[]  = {   0.0f ,  0.1f , 0.0f , -0.1f , 
                        0.0f , -0.1f , 0.0f ,  0.1f };
    
    memcpy(testVector->data[0], &buf  , 8);
    memcpy(testVector->data[1], &buf  , 8);
    
    
    testVectorTwo = new_fvec(8, 2);
    
    smpl_t buf2[]  = { 0.2f , 0.0f , -0.2f , 0.0f,
                      -0.2f , 0.015f ,  0.2f , 0.0f};
    
    memcpy(testVectorTwo->data[0], &buf2, 8);
    memcpy(testVectorTwo->data[1], &buf2, 8);
    
    testInterleavedList.mNumberBuffers = 1;
    
    int i;
    for (i = 0; i < testInterleavedList.mNumberBuffers; i++) {
        testInterleavedList.mBuffers[i].mNumberChannels = 2;
        testInterleavedList.mBuffers[i].mDataByteSize = sizeof(float) * 8;
        testInterleavedList.mBuffers[i].mData = malloc(testInterleavedList.mBuffers[i].mDataByteSize);
        
        float exampleData[] = { 0.0f , 0.0f,  0.1f * i, 0.1f * i, 
                                0.0f , 0.0f , -0.f * i, -0.1f * i };
        memcpy(testInterleavedList.mBuffers[i].mData, 
               exampleData, 
               testInterleavedList.mBuffers[i].mDataByteSize);
    }
    
    testDesc.mSampleRate = 48000.0f;
    testDesc.mFormatID = 'lpcm';
    testDesc.mFormatFlags = kLinearPCMFormatFlagIsFloat;
    testDesc.mBytesPerPacket = sizeof(float) *2;
    testDesc.mFramesPerPacket = 1;
    testDesc.mBytesPerFrame = testDesc.mBytesPerPacket;
    testDesc.mChannelsPerFrame = 2;
    testDesc.mBitsPerChannel = sizeof(float) * 8;
    
}

-(void)testZeroLength {
    ABOSampleVector *t = [[ABOSampleVector alloc] initWithLength:0 channelCount:1];
    
    STAssertNotNil(t,@"failed to create vector with zero length");
    STAssertEquals(t.length, (uint_t)0, @"failed to size vector to 0 length");
    
    ABOSampleVector *u = [[ABOSampleVector alloc] initWithLength:20 channelCount:1];
    ABOSampleVector *v = [t vectorByAppendingVector:u];
    
    STAssertNotNil(v,@"failed to append to vector with zero length");
    STAssertEquals(v.length, (uint_t)20, @"failed to append lengthy vector to vector of 0 length");
    
}

-(void)testAppend {
    
    fvec_t *aVector, *bVector;
    
    smpl_t bufa[] =  {0.1f, 0.2f, 0.3f, 0.4f}, bufb[] = {1.0f, 2.0f, 3.0f, 4.0f};
    
    aVector = new_fvec(4, 1);
    memcpy(aVector->data[0], bufa, sizeof(float) * 4);
    
    bVector = new_fvec(4, 1);
    memcpy(bVector->data[0], bufb, sizeof(float) * 4);
    
    ABOSampleVector *a = [[ABOSampleVector alloc] initWithFvec:aVector];
    
    ABOSampleVector *b = [[ABOSampleVector alloc] initWithFvec:bVector];
    
    ABOSampleVector *aPrime = [a vectorByAppendingVector:b];
    
    STAssertNotNil(aPrime,@"vectorByAppendingVector: returned nil");
    
    STAssertEquals(aPrime.channelCount, 
                   (uint_t)1, 
                   @"channel count of concat vector bad");
    
    STAssertEquals(aPrime.length, 
                   (uint_t)(a.length + b.length),
                   @"length of concat vector bad");
    
    smpl_t appendedBuf[] = { 0.1f, 0.2f, 0.3f, 0.4f,
        1.0f, 2.0f, 3.0f, 4.0f
                            };
    
    int i;
    for (i = 0; i < 8; i++) {
            STAssertEquals([aPrime sampleAtFrame:i inChannel:0], appendedBuf[i], 
                           @"append did not work, sample offset %i isn't what it should be",i);
    }
}

-(void)testVectorTrimming {
    smpl_t buffer[] = {1.0f, 1.5f, 2.0f, 2.5f, 3.0f};
    
    ABOSampleVector *t = [[ABOSampleVector alloc] initWithLength:5 
                                                    channelCount:1];
    
    [t putSamples:buffer withLength:5
         atOffset:0 inChannel:0];
    
    ABOSampleVector *tPrime = [t vectorByTrimmingFromSample:1 length:3];
    
    smpl_t compBuff[] = {1.5f, 2.0f, 2.5f};
    
    int i;
    for (i = 0; i < 3; i++) {
        STAssertEquals([tPrime sampleAtFrame:i inChannel:0], 
                       compBuff[i],
                       @"trimming didn't work");
    }
}


-(void)testCreateWithFVec {
    ABOSampleVector *t = [[ABOSampleVector alloc] initWithFvecNoCopy:testVector
                                                        freeWhenDone:NO];
    
    STAssertTrue(t.FVec->data != NULL,@"no copy failed");
    STAssertTrue(t.FVec == testVector,@"no copy failed");
    
    int i;
    for (i = 0; i < t.FVec->length; i++) {
        STAssertTrue(testVector->data[0][i] == t.FVec->data[0][i],@"data is not identical"); 
    }
    
    t = nil;
    ABOSampleVector *u = [[ABOSampleVector alloc] initWithFvec:testVectorTwo];
    
    STAssertTrue(u.FVec->data != NULL,@"copy failed");
    STAssertTrue(u.FVec != testVectorTwo,@"copy failed");
    
    for (i = 0; i < u.FVec->length; i++) {
        STAssertTrue(testVectorTwo->data[0][i] == u.FVec->data[0][i],@"data is not identical"); 
    }
}

-(void)testCopySamples {
    ABOSampleVector *t = [[ABOSampleVector alloc] initWithLength:8 channelCount:2];
    
    smpl_t buf1[] = {0.01f, -0.01f, 0.0f,  0.0f,  1.0f, -1.0f,  0.5f, -0.5f};
    smpl_t buf2[] = {0.08f,  0.07f, 0.06f, 0.05f, 0.04f, 0.03f, 0.02f, 0.01f};
    
    [t putSamples:buf1 withLength:8 inChannel:0];
    [t putSamples:buf2 withLength:8 inChannel:1];
    
    int i;
    for (i = 0; i < 8; i++) {
        STAssertTrue([t sampleAtFrame:i inChannel:0] == buf1[i],
                     @"");
        STAssertTrue([t sampleAtFrame:i inChannel:1] == buf2[i],
                     @"");
    }
}

-(void)testPutSamples {
    ABOSampleVector *t = [[ABOSampleVector alloc] initWithLength:8 channelCount:1];
    
    smpl_t buf1[] = {0.01f, -0.01f, 0.0f,  0.0f,  1.0f, -1.0f,  0.5f, -0.5f};
    smpl_t buf2[] = {0.05f, 0.04f, 0.03f};
    
    [t putSamples:buf1 withLength:8 inChannel:0];
    [t putSamples:buf2 withLength:3 atOffset:5 inChannel:0];
    
    smpl_t bufResult[] = {0.01f, -0.01f, 0.0f,  0.0f,  1.0f, 0.05f, 0.04f, 0.03f};
    
    int i;
    for (i = 0; i < 8; i++) {
        STAssertTrue([t sampleAtFrame:i inChannel:0] == bufResult[i],
                     @"");
    }
}

-(void)testAssertionFailureOnBounds {
    
    ABOSampleVector *t = [[ABOSampleVector alloc] initWithLength:10 channelCount:1];
    
    smpl_t *bigbuf = malloc(sizeof(smpl_t) * 11);
    
    
    STAssertThrows(
                   [t putSamples:bigbuf withLength:11 inChannel:0],
                   @"out-of-bounds sample put failed to throw exception");
    
    STAssertThrows(
                   [t putSamples:bigbuf withLength:1 atOffset:10 inChannel:0],
                   @"out-of-bounds sample put failed to throw exception");
    
}

-(void)testCreate {
    ABOSampleVector *t = [[ABOSampleVector alloc] initWithLength:8 channelCount:2];
    
    
    STAssertNotNil(t,@"-initWithLength:channelCount: returned nil");
    
    STAssertEquals(t.length, (uint_t)8, @"length is wrong");
    STAssertEquals(t.channelCount, (uint_t)2, @"length is wrong");
}

-(void)testCreateWithAudioBufferList {
    ABOSampleVector *t2 = [[ABOSampleVector alloc] initWithAudioBufferList:testInterleavedList audioStreamBasicDescription:testDesc];
    
    STAssertNotNil(t2,@"-[initWithAudioBufferList:] returned nil");
    
    STAssertEquals(t2.length, (uint_t)4, @"length is wrong");
    STAssertEquals(t2.channelCount, (uint_t)2,@"channel count is wrong");
}

-(void)testSubvectors {
    smpl_t buf[] = {0.1f, 0.2f, 0.3f, 0.4f, 0.5f, 0.6f, 0.7f, 0.8f, 0.9f, 1.0f};
    
    ABOSampleVector *t = [[ABOSampleVector alloc] initWithLength:10 channelCount:1];
    [t putSamples:buf withLength:10 inChannel:0];
    
    NSArray *result = [t subvectorsOfLength:2 period:3];
    
    STAssertEquals((NSUInteger)4, [result count],@"subvector count incorrect, should be 4");
    STAssertEquals((uint_t)2,(uint_t)[[result objectAtIndex:0] length],@"subvector 1 not right length, should be 2");
    STAssertEquals((uint_t)2,(uint_t)[[result objectAtIndex:1] length],@"subvector 2 not right length, should be 2");
    STAssertEquals((uint_t)2,(uint_t)[[result objectAtIndex:2] length],@"subvector 3 not right length, should be 2");
    STAssertEquals((uint_t)1,(uint_t)[[result objectAtIndex:3] length],@"subvector 4 not right length, should be 1");
    
    STAssertEquals((smpl_t)0.1f,(smpl_t)[[result objectAtIndex:0] sampleAtFrame:0 inChannel:0],@"subvector 1 not right content");
    STAssertEquals((smpl_t)0.4f,(smpl_t)[[result objectAtIndex:1] sampleAtFrame:0 inChannel:0],@"subvector 2 not right content");
    STAssertEquals((smpl_t)0.7f,(smpl_t)[[result objectAtIndex:2] sampleAtFrame:0 inChannel:0],@"subvector 3 not right content");
    STAssertEquals((smpl_t)1.0f,(smpl_t)[[result objectAtIndex:3] sampleAtFrame:0 inChannel:0],@"subvector 4 not right content");
    
}

-(void)tearDown {
    
    del_fvec(testVector);
    del_fvec(testVectorTwo);
    
    //free(testList);
}

@end
