//
//  ABOVectorEnumerator_testCase.m
//  Magi
//
//  Created by Jamie Hardt on 7/8/12.
//  Copyright (c) 2012 Jamie Hardt. All rights reserved.
//

#import "ABOVectorEnumerator_testCase.h"
#import "ABOVectorEnumerator.h"
#import "ABOSampleVector.h"

@implementation ABOVectorEnumerator_testCase

-(void)testEnumerateFile {
    AVAsset *oneKFile = [AVAsset assetWithURL:[[NSBundle bundleForClass:[self class]] URLForResource:@"1k Tone" withExtension:@"wav"]];
    
    AVAssetTrack *audioTrack = [[oneKFile tracksWithMediaType:AVMediaTypeAudio] lastObject];
    
    
    ABOVectorEnumerator *t = [[ABOVectorEnumerator alloc] initWithAsset:oneKFile
                                                                  track:audioTrack];
    
    __block NSUInteger totalSamples = 0;
    __block NSUInteger totalWindowsYielded = 0;
    [t enumerateSamplesWithBlock:^(ABOSampleVector *vector) {
        totalSamples += t.windowSize;
        totalWindowsYielded++;
        
        STAssertEquals(vector.length, (uint_t)512,@"enumerator did not yield the correct window size");
    } 
                    errorHandler:^(NSError *error) {
                        STAssertNotNil(nil,@"sample enumerator threw an error: %@",error);
                    }];
     
    const NSUInteger numberOfSamplesInFile = 240000;
    const NSUInteger correctNumberOfWindows = (numberOfSamplesInFile / t.periodLength);
    const NSUInteger correctTotalNumberOfSamples = correctNumberOfWindows * t.windowSize;
    
    STAssertEquals(correctNumberOfWindows, 
                   totalWindowsYielded,
                   @"failed to enumerate the proper number of window, should be %u",correctNumberOfWindows);
    
    STAssertEquals(correctTotalNumberOfSamples, 
                   totalSamples,
                   @"failed to enumerate the proper number of samples, should be %u",correctTotalNumberOfSamples);
}

// this test will require some more work, I might have to create a special audio file to test it.

//#define TESTBUFFSIZE (100)

//-(void)testEnumerateBufferInDetail {
//    smpl_t *buffer = malloc(TESTBUFFSIZE * sizeof(smpl_t));
//    
//    int i;
//    for (i = 0; i < TESTBUFFSIZE; i++) {
//        buffer[i] = 0.01f * i;
//    }
//    
//    ABOSampleVector *v = [[ABOSampleVector alloc] initWithLength:TESTBUFFSIZE 
//                                                    channelCount:1];
//
//    [v putSamples:buffer withLength:TESTBUFFSIZE inChannel:0];
//
//    
//}

@end
